package com.sumbytes.helloblog.posts.controller;

import com.sumbytes.common.annotation.Explain;
import com.sumbytes.common.annotation.LoginRequired;
import com.sumbytes.common.annotation.OperateLog;
import com.sumbytes.common.base.domain.Result;
import com.sumbytes.common.enums.OperateEnum;
import com.sumbytes.common.util.ThrowableUtils;
import com.sumbytes.helloblog.posts.domain.validator.CrawlerPosts;
import com.sumbytes.helloblog.posts.domain.validator.InsertPosts;
import com.sumbytes.helloblog.posts.domain.validator.UpdatePosts;
import com.sumbytes.helloblog.posts.domain.validator.UpdateStatus;
import com.sumbytes.helloblog.posts.domain.vo.PostsVO;
import com.sumbytes.helloblog.posts.service.PostsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


@RestController
@RequestMapping("/posts")
public class PostsController {

    @Autowired
    private PostsService postsService;

    @Explain("文章列表")
    @OperateLog(module = "文章列表", code=OperateEnum.GET_POSTS_LIST)
    @GetMapping("/posts/v1/list")
    public Result<PostsVO> getPostsList(PostsVO postsVO) {
        return postsService.getPostsList(postsVO);
    }

    @Explain("文章权重列表")
    @OperateLog(module = "权重列表", code=OperateEnum.GET_POSTS_LIST)
    @GetMapping("/weight/v1/list")
    public Result<PostsVO> getWeightList(PostsVO postsVO) {
        postsVO.setIsWeight(1);
        return postsService.getPostsList(postsVO);
    }

    @Explain("按照时间归档文章数量")
    @GetMapping("/archive/v1/list")
    public Result<PostsVO> getArchiveTotalByDateList(PostsVO postsVO) {
        return postsService.getArchiveTotalByDateList(postsVO);
    }

    @Explain("查询热门文章")
    @GetMapping("/hot/v1/list")
    public Result getHotPostsList(PostsVO postsVO) {
        return postsService.getHotPostsList(postsVO);
    }

    @OperateLog(module = "获取文章", code=OperateEnum.GET_POSTS_DETAIL)
    @GetMapping("/posts/v1/{id}")
    public Result getPosts(@PathVariable Long id) {
        return this.postsService.getPosts(id);
    }


    @Explain("查询文章统计")
    @GetMapping("/posts/v1/statistics")
    public Result getPostStatistics(){
        return this.postsService.getPostStatistics();
    }


    @Explain("添加文章")
    @LoginRequired
    @PostMapping("/posts/v1/add")
    public Result savePosts(@Validated({InsertPosts.class}) @RequestBody PostsVO postsVO, BindingResult result) {
        ThrowableUtils.checkParamArgument(result);
        return postsService.savePosts(postsVO);
    }

    @Explain("删除文章")
    @LoginRequired
    @DeleteMapping("/posts/v1/{id}")
    public Result deletePosts(@PathVariable Long id) {
        return this.postsService.deletePosts(id);
    }

    @Explain("修改文章")
    @PutMapping("/posts/v1/update")
    @LoginRequired
    public Result updatePosts(@Validated({UpdatePosts.class}) @RequestBody PostsVO postsVO, BindingResult result) {
        ThrowableUtils.checkParamArgument(result);
        return this.postsService.updatePosts(postsVO);
    }

    @Explain("修改文章状态")
    @PutMapping("/status/v1/update")
    @LoginRequired
    public Result updatePostsStatus(@Validated({UpdateStatus.class}) @RequestBody PostsVO postsVO, BindingResult result) {
        ThrowableUtils.checkParamArgument(result);
        return this.postsService.updatePostsStatus(postsVO);
    }

    @Explain("博客搬家")
    @PostMapping("/posts/v1/crawler")
    @LoginRequired
    public Result getPlatformArticle(@Validated({CrawlerPosts.class}) @RequestBody PostsVO postsVO, BindingResult result) {
        ThrowableUtils.checkParamArgument(result);
        return this.postsService.getPlatformArticle(postsVO);
    }

}
